import java.io.*;
import java.util.*;
import org.chocosolver.solver.Model;
import org.chocosolver.solver.Solver;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.constraints.IIntConstraintFactory.*;
import org.chocosolver.solver.search.strategy.Search;
import org.chocosolver.solver.search.strategy.strategy.IntStrategy;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainMax;
import org.chocosolver.solver.exception.ContradictionException;

public class Optimize {    
  
  public static void main(String[] args) throws FileNotFoundException, IOException {
      int timeLimit         = Integer.parseInt(args[1]);
      JSSP jssp             = new JSSP(args[0],9999);
      Model model           = jssp.model;
      Solver solver         = model.getSolver();
      Decision[] decisions  = jssp.getDecisions();
      int n                 = decisions.length;
      IntVar makeSpan       = jssp.getMakeSpan();

      solver.limitTime(timeLimit*1000);

      solver.setSearch(new IntStrategy(jssp.getDecisionIntVars(),new MinSlackHeuristic(jssp.getDecisions()),new IntDomainMax()));
      //
      // attach a variable & value ordering heuristic to solver
      //

      // alternative way to optimize
      // Solution solution = solver.findOptimalSolution(makeSpan,false);

      // trace optimisation
      model.setObjective(Model.MINIMIZE,makeSpan);
      while(solver.solve()) System.out.println("makespan: ["+ makeSpan.getLB() +","+ makeSpan.getUB() +"]" +
					       "   nodes: "+ solver.getMeasures().getNodeCount());

      System.out.println("nodes: "+ solver.getMeasures().getNodeCount() + 
                         "  cpu: "+ solver.getMeasures().getTimeCount());
  }
}